<?php
include_once('settings.php');

$elvantotools_last_error = '';

function elvantotools_sort_by_array($src, $keys) {
	$dest = array();
	foreach ($keys as $key)
		if (array_key_exists($key, $src))
			$dest[$key] = $src[$key];
	return $dest;
}

function elvantotools_curl_cache($url, $fields, $api_key) {
	global $elvantotools_last_error;
	
	$elvantotools_last_error = '';
	
	$fields_string = '';
	foreach($fields as $key=>$value) { $fields_string .= $key.'='.$value.'&'; }
	rtrim($fields_string, '&');
	
	$parms = md5(serialize(array($url, $api_key, $fields_string)));
	$plugin_cache = plugin_dir_path( __FILE__ ) . 'cache';
	$cache_result = trailingslashit($plugin_cache) . $parms . '.dat'; 
	
	if (file_exists($plugin_cache)) {
		// Delete any cache files more than a week old
		$files = glob(trailingslashit($plugin_cache) . "*.dat");
		$now = time();
		foreach ($files as $file) {
			if (is_file($file) && $file != $cache_result)
				if ($now - filemtime($file) >= 60*60*24)
					unlink($file);
		}
		// If the cache file exists return the contents
		if ((!isset($_GET['sp']) || $_GET['sp'] != 'flush') && file_exists($cache_result)) {
			return file_get_contents($cache_result);
		}
	} else {
		if (!mkdir($plugin_cache)) {
			$elvantotools_last_error = 'Failed to create cache directory';
			return FALSE;
		}
	}
	
	// Otherwise we'll need to download the data
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_POST, count($fields));
	curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
	curl_setopt($ch, CURLOPT_USERPWD, $api_key.":x");
	curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
	$output = curl_exec($ch);
	if ($output === FALSE) {
		$elvantotools_last_error = 'Failed to retrieve data: CURL failure';
		return FALSE;
	}
	$info = curl_getinfo($ch);
	curl_close($ch);
	
	if ($info['http_code'] == 200) {
		file_put_contents($cache_result, $output);
		return $output;
	} else {
		$elvantotools_last_error = 'Failed to retrieve data: Server returned code '.$info['http_code'];
		error_log($elvantotools_last_error);
		return FALSE;
	}
}

function elvantotools_roster_generate ($api_key, $offset, $weeks, $service_max, $service_types, $departments, $department_subs, $department_positions, $department_positions_merge, $columns, $dates, $zone, $sort, $prefix, $suffix, $top_left_content,  $header_label_content, $header_content, $header_prefix, $header_suffix, $header_element_prefix, $header_element_suffix, $body_prefix, $body_suffix, $row_prefix, $row_suffix, $label_content, $element_prefix, $element_suffix, $name_prefix, $name_suffix, $name_separator, $content, $empty_prefix, $empty_suffix, $show_details) {
	
	//error_log('label_content='.$label_content);
	//error_log('departments='.print_r($departments,true));
	//error_log('department_subs='.print_r($department_subs,true));
	//error_log('department_positions='.print_r($department_positions,true));
	
	$old_tz = date_default_timezone_get();
	date_default_timezone_set($zone);
	$service_types = trim($service_types) ? explode(',',$service_types) : array();
	
	$exclude_departments = (substr($departments,0,1) == '!');
	if ($exclude_departments) 
		$departments = substr($departments,1);
	$exclude_sub_departments = (substr($department_subs,0,1) == '!');
	if ($exclude_sub_departments)
		$department_subs = substr($department_subs,1);
	$exclude_department_positions = (substr($department_positions,0,1) == '!');
	if ($exclude_department_positions) 
		$department_positions = substr($department_positions,1);	
	
	//error_log('departments='.print_r($departments,true));
	//error_log('department_subs='.print_r($department_subs,true));
	//error_log('department_positions='.print_r($department_positions,true));	
	
	$departments = trim($departments) ? explode(',',$departments) : array();
	$department_subs = trim($department_subs) ? explode(',',$department_subs) : array();
	$department_positions = trim($department_positions) ? explode(',',$department_positions) : array();
	
	// Parse any position merging or renaming that needs to be done into a simple mapping of positions to their new names
	$merged_positions = trim($department_positions_merge) ? explode(',',$department_positions_merge) : array();
	$merged = array();
	foreach ($merged_positions as $merged_position) {
		if (count($merged_sides = explode('=', $merged_position)) == 2) {
			if (count($merged_names = explode('+', $merged_sides[0])) >= 1) {
				foreach ($merged_names as $merged_name) {
					$merged[trim($merged_name)] = trim($merged_sides[1]);
				}
				// We also need to add the merged/renamed column to our position filter if there is one
				if (count($department_positions) && !$exclude_department_positions)
					$department_positions[] = trim($merged_sides[1]);
			}
		}
	}
	
	// Get the service data from Elvanto
	$cur_time = new DateTime();
	$next_time = new DateTime();
	if ($offset + $weeks >= 0) {
		$cur_time->add(new DateInterval('P' . strval($offset) . 'W'));
		$next_time->add(new DateInterval('P' . strval($offset + $weeks) . 'W'));
	} else {
		$next_time->add(new DateInterval('P' . strval($offset) . 'W'));
		$cur_time->sub(new DateInterval('P' . strval(abs($offset + $weeks)) . 'W'));
	}
	
	$fields = array(
		'start' => urlencode($cur_time->format('Y-m-d')),
		'end' => urlencode($next_time->format('Y-m-d')),
		'fields[0]' => urlencode('volunteers')
	);

	if ($show_details) {
		$fields['fields[1]'] = urlencode('series_name');
		$fields['fields[2]'] = urlencode('plans');
	}
	
	global $elvantotools_last_error;
	if (($output = elvantotools_curl_cache("https://api.elvanto.com/v1/services/getAll.json", $fields, $api_key)) === FALSE) {
		$output = '<p>'.$elvantotools_last_error.'</p>';
		date_default_timezone_set($old_tz);
		return $output;		
	}
	
	$jdata = json_decode($output, true);
	//error_log('jdata='.print_r($jdata,true));
	$service_dates = array();
	$depts = array();
	$service_plans = array();
	$service_series = array();
	$service_ids = array();
	
	$show_depts = (strpos($label_content, '[department]') !== FALSE);
	$show_subs = (strpos($label_content, '[sub_department]') !== FALSE);
	$show_labels = ($label_content != '');
	
	if (array_key_exists('error', $jdata)) {
		$output = '<p>'.$jdata['error']['message'].'</p>';
		date_default_timezone_set($old_tz);
		return $output;
	}
	//error_log(print_r($jdata,true));<br>

	$service_count = 0;
	foreach ($jdata['services']['service'] as $service) {
	  if (!count($service_types) || in_array($service['service_type']['name'], $service_types)) {
		 $service_count++;
		 if ($service_max && $service_count > $service_max)
			 break;
		 $service_date = '&nbsp;';
	 	 try {
			$t = date_create_from_format('Y-m-d G:i:s', $service['date'], new DateTimeZone('UTC'));
			$t->setTimeZone(new DateTimeZone($zone));
			if ($t !== FALSE) {
				$service_date = str_replace(' ', '&nbsp;', $t->format($dates));
			}
		 } catch (Exception $e) {
		 }
		  
 		 if (!array_key_exists($service_date, $service_dates)) {
			$service_dates[$service_date] = array();
			$service_plans[$service_date] = array();
			$service_series[$service_date] = array();
			$service_ids[$service_date] = array();
		 }
		  
		 if ($service['location']['name'])
			$service_name = $service['location']['name'] . ' (' . $service['service_type']['name'] . ')';
		 else
		 	$service_name = $service['service_type']['name'];
		 $service_dates[$service_date][] = $service_name;
		  
		 if ($show_details) {
			$service_series[$service_date][$service_name] = $service['series_name'];
			$service_ids[$service_date][$service_name] = $service['id'];
			if (isset($service['plans']) && is_array($service['plans']) && isset($service['plans']['plan']) && is_array($service['plans']['plan'])) {
				foreach ($service['plans']['plan'] as $plan) {
					//error_log('plan='.print_r($plan,true));
					$new_plan = array();
					foreach ($plan['items']['item'] as $item) {
						$new_item = array();
						$new_item['duration'] = $item['duration'];
						$new_item['title'] = $item['title'];
						$new_item['type'] = $item['heading'] ? 'heading' : ($item['song'] ? 'song' : 'item');
						$new_item['description'] = $item['description'];
						$new_plan[] = $new_item;
					}				
					$service_plans[$service_date][$service_name] = $new_plan;
				}
			}
		 }
		  
		  
		 foreach ($service['volunteers']['plan'] as $plan) {
		   foreach ($plan['positions']['position'] as $position) {
			   //error_log('test position[sub_department_name]='.$position['sub_department_name']);
				//error_log('test department_subs='.print_r($department_subs,true));
			   //error_log('test exclude_sub_departments='.($exclude_sub_departments ? 'true' : 'false'));
			   //error_log('test false='.print_r(false,true));
			 if ((!count($departments) || ($exclude_departments xor in_array($position['department_name'], $departments))) &&
				 (!count($department_subs) || ($exclude_sub_departments xor in_array($position['sub_department_name'], $department_subs))) &&
				 (!count($department_positions) || ($exclude_department_positions xor in_array($position['position_name'], $department_positions)))) {
				 //error_log('passed');
			   if (is_array($position['volunteers'])) {
				 if (!$show_depts) $position['department_name'] = 'All';
				 if (!$show_subs) $position['sub_department_name'] = 'All';
				 if (!array_key_exists($position['department_name'], $depts))
					 $depts[$position['department_name']] = array();
				 if (!array_key_exists($position['sub_department_name'], $depts[$position['department_name']]))
					 $depts[$position['department_name']][$position['sub_department_name']] = array();
				 // Here we allow remapping of the position name for merging or renaming of positions
				 $position_name = array_key_exists($position['position_name'],$merged) ? $merged[$position['position_name']] : $position['position_name'];			   
				 if (!array_key_exists($position_name, $depts[$position['department_name']][$position['sub_department_name']]))
				   $depts[$position['department_name']][$position['sub_department_name']][$position_name] = array();
				 if (!array_key_exists($service_date, $depts[$position['department_name']][$position['sub_department_name']][$position_name]))
				    $depts[$position['department_name']][$position['sub_department_name']][$position_name][$service_date] = array();
				 if (!array_key_exists($service['service_type']['name'], 
									   $depts[$position['department_name']][$position['sub_department_name']][$position_name][$service_date]))
				 	$depts[$position['department_name']][$position['sub_department_name']][$position_name][$service_date][$service['service_type']['name']] = array();
				 foreach ($position['volunteers']['volunteer'] as $person) {
				   if (array_key_exists('firstname',$person['person']) && count($person['person']['firstname']))
					   $person['person']['firstinitial'] = substr(ucfirst($person['person']['firstname']),0,1) . '.';
				   if (array_key_exists('lastname',$person['person']) && count($person['person']['lastname']))
					   $person['person']['lastinitial'] = substr(ucfirst($person['person']['lastname']),0,1) . '.';
				   $depts[$position['department_name']][$position['sub_department_name']][$position_name][$service_date][$service_name][] = $person['person'];
				 }
			   }
			 }
		   }
		 }
	  }
	}
	
	//error_log('depts='.print_r($depts,true));
	// Display the headers
	$table_head = '';
	if ($header_content) {
		$table_head .= $header_prefix . $row_prefix;
		
		if (!$top_left_content) $top_left_content = '&nbsp;';
		
		$table_head .= str_replace(array('[department]','[sub_department]','[position]'),
								   array($show_depts ? $top_left_content : '&nbsp;',
										 (!$show_depts && $show_subs) ? $top_left_content : '&nbsp;',
										 !($show_depts || $show_subs) ? $top_left_content : '&nbsp;'),				
								   $header_label_content);
		
		foreach ($service_dates as $service_date=>$services) {
			reset($services); $first_service = current($services);
			foreach ($services as $service) {
				if ($service == $first_service)
					$table_head .= $header_element_prefix.str_replace('[service]',$service_date,$header_content).$header_element_suffix;
				else
					$table_head .= $header_element_prefix.str_replace('[service]','&nbsp;',$header_content).$header_element_suffix;					
			}
		}
		
		
		$table_head .= $row_suffix . $row_prefix;
		
		$table_head .= str_replace(array('[department]','[sub_department]','[position]'),
								   array('Department','Sub-department','Position'),				
								   $header_label_content);
		
		if ($show_details)
			$details = '&nbsp;<a class="elvantotools-popup-trigger" href="#[id]"><span class="dashicons dashicons-format-aside"></span></a><div id="[id]" class="elvantotools-overlay"><div class="elvantotools-popup"><a class="elvantotools-popup-close" href="#" >&times;</a><div class="elvantotools-popup-content">[plan]</div></div></div>';
		
		//error_log('service_dates='.print_r($service_dates,true));
		
		foreach ($service_dates as $service_date=>$services) {
			foreach ($services as $service) {
				if ($show_details && isset($service_plans[$service_date]) && is_array($service_plans[$service_date]) && 
				   					 isset($service_plans[$service_date][$service]) && is_array($service_plans[$service_date][$service])) {
					$formatted_details = '<table class="elvantotools-service-plan"><thead><tr><th class="elvantotools-series-name" colspan="2">'.$service_series[$service_date][$service].'</th></tr></thead><tbody>';
					foreach ($service_plans[$service_date][$service] as $item) {
						$formatted_details .= '<tr class="elvantotools-item elvantotools-item-'.$item['type'].'">
													<td><div class="elvantotools-item-title">'.$item['title'].'</div>'.($item['description'] ? '<div class="elvantotools-item-description">'.$item['description'].'</div>':'').'</td>
													<td class="elvantotools-item-duration">'.$item['duration'].'</td>
											   </tr>';
					}
					$formatted_details .= '</tbody></table>';
					$formatted_details = str_replace('[plan]',$formatted_details,$details);					
					$formatted_details = str_replace('[id]',$service_ids[$service_date][$service],$formatted_details);					
				} else {
					$formatted_details = '';
				}
				$table_head .= $header_element_prefix.str_replace('[service]',$service.$formatted_details,$header_content).$header_element_suffix;
			}
		}
		
		$table_head .= $row_suffix . $header_suffix;
	}

	// Pre-find all the substitutions and locations in the content to be repeated for each row
	$matches = array();
	preg_match_all('/[\[{]('.str_replace(',','|',ELVANTOTOOLS_ROSTER_FIELD_NAMES).')[\]}]/i', $content, $matches, PREG_OFFSET_CAPTURE);

	//error_log('label_content='.$label_content);
	//error_log('content='.$content);
	

	//	error_log('content='.print_r($content, true));
//	error_log(print_r($matches, true));
	$row_odd = false; $row_class = 'elvantotools_row_even';
	$table_body = $body_prefix;
	// Sort the array if a list of deparetments was specified
	if (count($departments) && !$exclude_departments) {
		$depts = elvantotools_sort_by_array($depts, $departments);
		//error_log('depts='.print_r($depts,true));
	}
	reset($depts); $first_dept = key($depts);	
	foreach ($depts as $dept=>$subs) {
		//error_log('subs='.print_r($subs,true));
		if (count($department_subs) && $show_subs && !$exclude_sub_departments)
			$subs = elvantotools_sort_by_array($subs, $department_subs);
		//error_log('subs='.print_r($subs,true));
		reset($subs); $first_sub = key($subs);
		foreach ($subs as $sub=>$positions) {
			if (count($department_positions) && !$exclude_department_positions)
				$positions = elvantotools_sort_by_array($positions, $department_positions);			
			reset($positions); $first_pos = key($positions);	
			foreach ($positions as $position=>$position_dates) {
				$col_odd = false; $col_class = 'elvantotools_col_even';
				if (($position == $first_pos && ($show_depts || $show_subs)) || !($show_depts || $show_subs)) {
					$row_odd = !$row_odd; $row_class = ($row_odd ? 'elvantotools_row_odd' : 'elvantotools_row_even');
				}
				$classes = $row_class . ' ' . $col_class;
				$table_body .= $row_prefix;
				if ($label_content)
					$table_body.=str_replace(array('[department]','[sub_department]','[position]'),
											 array(($sub == $first_sub && $position == $first_pos) ? $dept : '&nbsp;',
												   ($position == $first_pos) ? $sub : '&nbsp;',
												   $position),		
											 str_replace('[class]',$classes, $label_content));
				foreach ($service_dates as $service_date=>$services) {
					$col_odd = !$col_odd; $col_class = ($col_odd ? 'elvantotools_col_odd' : 'elvantotools_col_even');
					$classes = $row_class . ' ' . $col_class;
					foreach ($services as $service_name) {
						$table_body .= str_replace('[class]',$classes, $element_prefix);
						if (array_key_exists($service_date, $position_dates) && array_key_exists($service_name, $position_dates[$service_date])) {
							for ($i = 0; $i < count($position_dates[$service_date][$service_name]); $i+=1) {
								if ($i)
									$table_body .= $name_separator;
								$person = $position_dates[$service_date][$service_name][$i];
								$table_body .= str_replace('[id]', $person['id'], $name_prefix);
								if (isset($matches[1])) {
									$row = $content;
									// work through in reverse so substitutions do not affect earlier offsets
									for ($f = count($matches[1]) - 1; $f >= 0; $f--) {
										$text = '&nbsp;';
										$col = $matches[1][$f][0];
										if (array_key_exists($col, $person)) {
											$text = $person[$col];
										}
										//error_log('subbing "'.$col.'" as "'.$text.'"');
										$row = substr_replace($row, $text, $matches[1][$f][1] - 1, strlen($col) + 2);
									}
									$table_body .= $row;
								} else {
									$table_body .= $content;
								}
								$table_body .= $name_suffix;
							}
						} else {
							$table_body .= '&nbsp;';
						}
						$table_body .= $element_suffix;
					}
				}
				$table_body .= $row_suffix;		
			}
		}
	}
	
	if (!count($depts)) {
		$table_body .= $empty_prefix . 'Nothing found' . $empty_suffix;
	}
	
	$table_body .= $body_suffix;
	
	$output = $prefix . $table_head . $table_body . $suffix;
	
	date_default_timezone_set($old_tz);
	
	return $output;
}

function elvantotools_songs_generate ($api_key, $offset, $weeks, $service_max, $service_types, $categories, $columns, $dates, $zone, $sort, $prefix, $suffix, $top_left_content, $header_label_content, $header_content, $header_prefix, $header_suffix, $header_element_prefix, $header_element_suffix, $body_prefix, $body_suffix, $row_prefix, $row_suffix, $label_content, $element_prefix, $element_suffix, $name_prefix, $name_suffix, $name_separator, $content, $empty_prefix, $empty_suffix, $video_width, $video_height, $in_plan) {
	
	//error_log('video_width='.$video_width.',video_height='.$video_height);
	$old_tz = date_default_timezone_get();
	date_default_timezone_set($zone);
	$service_types = trim($service_types) ? explode(',',$service_types) : array();
	$categories = trim($categories) ? explode(',',$categories) : array();
	
	// Get the service data from Elvanto
	$cur_time = new DateTime();
	$next_time = new DateTime();
	if ($offset + $weeks >= 0) {
		$cur_time->add(new DateInterval('P' . strval($offset) . 'W'));
		$next_time->add(new DateInterval('P' . strval($offset + $weeks) . 'W'));
	} else {
		$next_time->add(new DateInterval('P' . strval($offset) . 'W'));
		$cur_time->sub(new DateInterval('P' . strval(abs($offset + $weeks)) . 'W'));
	}

	$fields = array(
		'start' => urlencode($cur_time->format('Y-m-d')),
		'end' => urlencode($next_time->format('Y-m-d')),
		'fields[0]' => urlencode('songs')
	);

	if ($in_plan)
		$fields['fields[1]'] = urlencode('plans');
	
	global $elvantotools_last_error;
	if (($output = elvantotools_curl_cache("https://api.elvanto.com/v1/services/getAll.json", $fields, $api_key)) === FALSE) {
		$output = '<p>'.$elvantotools_last_error.'</p>';
		date_default_timezone_set($old_tz);
		return $output;		
	}
	
	$jdata = json_decode($output, true);
	//error_log('jdata='.print_r($jdata,true));
	$service_dates = array();
	$cats = array();
	$songs = array();
	$playlists = array();
	
	$show_labels = ($label_content != '');
	
	if (array_key_exists('error', $jdata)) {
		$output = '<p>'.$jdata['error']['message'].'</p>';
		date_default_timezone_set($old_tz);
		return $output;
	}
	//error_log(print_r($jdata,true));<br>

	$song_count = 0;
	foreach ($jdata['services']['service'] as $service) {
	  if (!count($service_types) || in_array($service['service_type']['name'], $service_types)) {
		 $song_count++;
		 if ($service_max && $song_count > $service_max)
			 break;
		 $service_date = '&nbsp;';
	 	 try {
			$t = date_create_from_format('Y-m-d G:i:s', $service['date'], new DateTimeZone('UTC'));
			$t->setTimeZone(new DateTimeZone($zone));
			if ($t !== FALSE) {
				$service_date = str_replace(' ', '&nbsp;', $t->format($dates));
			}
		 } catch (Exception $e) {
		 }
		  
 		 if (!array_key_exists($service_date, $service_dates)) {
			$service_dates[$service_date] = array();
			$playlists[$service_date] = array();
		 }
		  
		 if ($service['location']['name'])
			$service_name = $service['location']['name'] . ' (' . $service['service_type']['name'] . ')';
		 else
		 	$service_name = $service['service_type']['name'];
		 $service_dates[$service_date][] = $service_name;
		 $playlists[$service_date][$service_name] = array();
		  
		 if (!array_key_exists($service_date, $songs))
			$songs[$service_date] = array();
		 $songs[$service_date][$service_name] = array();
		  
		 $plan_songs = array();
		 if ($in_plan && 
			 isset($service['plans']) && is_array($service['plans']) && 
			 isset($service['plans']['plan']) && is_array($service['plans']['plan']) && count($service['plans']['plan']) && 
			 isset($service['plans']['plan'][0]['items']) && is_array($service['plans']['plan']['0']['items']) &&
			 isset($service['plans']['plan'][0]['items']['item']) && is_array($service['plans']['plan']['0']['items']['item'])
			) {
			 foreach ($service['plans']['plan'][0]['items']['item'] as $item) {
				 if (isset($item['song']) && is_array($item['song'])) {
					 if (isset($item['song']['id']))
						 $plan_songs[] = $item['song']['id'];
				 }
			 }
		 }
		  //error_log('plan_songs='.print_r($plan_songs, true));
		  
		 if (isset($service['songs']) && is_array($service['songs']) && isset($service['songs']['song']) && is_array($service['songs']['song'])) {
		  foreach ($service['songs']['song'] as $song) {
			if ($in_plan && ($song_index = array_search($song['id'], $plan_songs)) === FALSE)
				continue;
			//error_log('song_id='.$song['id']);
			//error_log('song_index='.$song_index);
			//error_log('song='.print_r($song,true));
			if (count($categories) || preg_match('/\[(video_link|video_embed|audio_link|image_link)\]/', $content)) {
				if (($output = elvantotools_curl_cache("https://api.elvanto.com/v1/songs/getInfo.json", 
										 array('id' => urlencode($song['id']), 'files'=>true), 
										 $api_key)) !== FALSE) {
				   $jsong = json_decode($output, true);
				   if (array_key_exists('error', $jsong)) {
						error_log('Failed to get song: '.$jsong['error']['message']);
				   } else if (!array_key_exists('song',$jsong) || !is_array($jsong['song']) || !count($jsong['song'])) {
					   error_log('Failed to find song in server response');
				   } else {
					//error_log('song='.print_r($jsong,true));
					   $song['categories'] = array();
					   if (isset($jsong['song'][0]['categories']) && is_array($jsong['song'][0]['categories']) && 
						   isset($jsong['song'][0]['categories']['category']) && is_array($jsong['song'][0]['categories']['category'])) {
						   foreach ($jsong['song'][0]['categories']['category'] as $category) {
							   $cats[] = $category['name'];
							   $song['categories'][] = $category['name'];
						   }
					   }
					   if (count($categories) && !count(array_intersect($categories, $song['categories'])))
						   continue;
					   else
						   $song['categories'] = implode(',',$song['categories']);
					   if (isset($jsong['song'][0]['files']) && is_array($jsong['song'][0]['files']) &&
						   isset($jsong['song'][0]['files']['file']) && is_array($jsong['song'][0]['files']['file'])) {
						   foreach ($jsong['song'][0]['files']['file'] as $file) {
							   switch ($file['type']) {
								   case 'Video':
									   $song['video_embed'] = $file['content'];
									   // Flow on intentional									   
								   case 'Image':
								   case 'Audio':
									   $song[strtolower($file['type']).'_link'] = $file['content'];
									   break;
							   }
						   }
					   }
				   }
			   }			   
		   }
		   if (isset($song['arrangement'])) {
			   $song['arrangement_title'] = $song['arrangement']['title'];
			   if (count($categories) || preg_match('/\[(copyright|first_line|video_link|video_embed|audio_link|image_link)\]/', $content)) {  
				   if (($output = elvantotools_curl_cache("https://api.elvanto.com/v1/songs/arrangements/getInfo.json", 
											 array('id' => urlencode($song['arrangement']['id']), 'files'=>true), 
											 $api_key)) !== FALSE) {
					   $jsong = json_decode($output, true);
					   if (array_key_exists('error', $jsong)) {
							error_log('Failed to get arrangement: '.$jsong['error']['message']);
					   } else if (!array_key_exists('arrangement',$jsong) || !is_array($jsong['arrangement']) || !count($jsong['arrangement'])) {
							error_log('Failed to find arrangement in server response');
					   } else {
						   //error_log('arrangement='.print_r($jsong,true));
						   if (isset($jsong['arrangement'][0]['copyright']) && trim($jsong['arrangement'][0]['copyright'])) 
							   $song['copyright'] = $jsong['arrangement'][0]['copyright'];
						   if (isset($jsong['arrangement'][0]['lyrics'])) {
							   $lines = explode("\n",$jsong['arrangement'][0]['lyrics']);
							   //error_log('lines='.print_r($lines,true));
							   while (count($lines) && (!trim($lines[0]) || in_array(substr(trim(strtolower($lines[0])),0,5),array('verse','choru','intro')))) 	
								   array_shift($lines);
							   if (count($lines)) $song['first_line'] = trim($lines[0]);
						   }
						   if (isset($jsong['arrangement'][0]['files']) && is_array($jsong['arrangement'][0]['files']) &&
							   isset($jsong['arrangement'][0]['files']['file']) && is_array($jsong['arrangement'][0]['files']['file'])) {
							   foreach ($jsong['arrangement'][0]['files']['file'] as $file) {
								   switch ($file['type']) {
									   case 'Video':
										   $song['video_embed'] = $file['content'];
										   // Flow on intentional
									   case 'Image':
									   case 'Audio':
										   $song[strtolower($file['type']).'_link'] = $file['content'];
										   break;
								   }
							   }
						   }
					   }
				   }
			   }
		   }
		   if (isset($song['ccli_number']) && trim($song['ccli_number']))
			   $song['audio_sample'] = $song['ccli_number'];
		   if (isset($song['video_link'])) {
			   $v = array();
			   if (preg_match('/http(?:s?):\/\/(?:www\.)?youtu(?:be\.com\/watch\?v=|\.be\/)([\w\-\_]*)(&(amp;)?[\w\?=]*)?/', $song['video_link'], $v)) {
			      $song['video_youtube'] = $v[1];
				  $playlists[$service_date][$service_name][] = $song['video_youtube'];
			   }
		   }
		   if ($in_plan)
			  $songs[$service_date][$service_name][$song_index] = $song;
		   else
		   	  $songs[$service_date][$service_name][] = $song;		   
		 }
		}
	  }
	}
	
	//error_log('songs='.print_r($songs,true));
	// Display the headers
	$table_head = '';
	if ($header_content) {
		$table_head .= $header_prefix . $row_prefix;
				
		$table_head .= $header_label_content;
		reset($service_dates); $first_date = key($service_dates);
		foreach ($service_dates as $service_date=>$services) {
			reset($services); $first_service = current($services);
			foreach ($services as $service) {
				if ($service == $first_service)
					$table_head .= $header_element_prefix.str_replace('[service]',($service_date == $first_date ? $top_left_content . '&nbsp;' : ''). $service_date,$header_content).$header_element_suffix;
				else
					$table_head .= $header_element_prefix.str_replace('[service]','&nbsp;',$header_content).$header_element_suffix;					
			}
		}
		
		$table_head .= $row_suffix . $row_prefix;
		
		$table_head .= str_replace('[song]','Song',$header_label_content);

		foreach ($service_dates as $service_date=>$services) {
			foreach ($services as $service) {
				if (count($playlists[$service_date][$service]))
					$playlist = '&nbsp;<a target="_blank" href="https://www.youtube.com/watch_videos?video_ids='.implode(',',$playlists[$service_date][$service]).'"><span class="dashicons dashicons-video-alt2"></span></a>';
				else
					$playlist = '';
				$table_head .= $header_element_prefix.str_replace('[service]',$service.$playlist,$header_content).$header_element_suffix;
			}
		}
		
		$table_head .= $row_suffix . $header_suffix;
	}

	// Pre-find all the substitutions and locations in the content to be repeated for each row
	$matches = array();
	preg_match_all('/[\[{]('.str_replace(',','|',ELVANTOTOOLS_SONGS_FIELD_NAMES).')[\]}]/i', $content, $matches, PREG_OFFSET_CAPTURE);

	//error_log('label_content='.$label_content);
	//error_log('content='.$content);
	
	//	error_log('content='.print_r($content, true));
//	error_log(print_r($matches, true));
	$row_odd = false; $row_class = 'elvantotools_row_even';
	$table_body = $body_prefix;	
	$classes = $row_class . ' ' . $col_class;
	$table_body .= $row_prefix;
	
	if ($label_content)
		$table_body.=str_replace(array('[song]'),'&nbsp;',str_replace('[class]',$classes, $label_content));
	
	foreach ($service_dates as $service_date=>$services) {
		$col_odd = !$col_odd; $col_class = ($col_odd ? 'elvantotools_col_odd' : 'elvantotools_col_even');
		$classes = $row_class . ' ' . $col_class;
		foreach ($services as $service_name) {
			$table_body .= str_replace('[class]',$classes, $element_prefix);
			for ($i = 0; $i < count($songs[$service_date][$service_name]); $i+=1) {
				if ($i)
					$table_body .= $name_separator;
				$song = $songs[$service_date][$service_name][$i];
				//error_log('song='.print_r($song, true));
				$table_body .= str_replace('[id]', $song['id'], $name_prefix);
				if (isset($matches[1])) {
					$row = $content;
					// work through in reverse so substitutions do not affect earlier offsets
					for ($f = count($matches[1]) - 1; $f >= 0; $f--) {
						$text = '';
						$col = $matches[1][$f][0];
						if (array_key_exists($col, $song)) {
							switch ($col) {
								case 'video_embed':
									if (substr($song[$col],0,1) == '<') {
										$text = $song[$col];	
									} else {
										if (isset($song['video_youtube'])) {
											$text = '[youtube https://www.youtube.com/watch?v='.$song['video_youtube'];
											if ($video_width) $text .= '&w='.$video_width.'&h='.$video_height;
											$text .= ' ]';
										} else {
											$text = '<a target="_blank" href="'.$song[$col].'">Watch</a>';
										}
									}
									break;
								case 'video_link':
									if (trim($song[$col]))
										$text = '<a target="_blank" href="'.$song[$col].'"><span class="dashicons dashicons-video-alt2"></span></a>';
									break;
								case 'audio_link':
									if (trim($song[$col]))
										$text = '<a target="_blank" href="'.$song[$col].'"><span class="dashicons dashicons-format-audio"></span></a>';
									break;
								case 'audio_sample':
										$text = '<a class="elvantotools_play" href="https://d35zixk8uui5fz.cloudfront.net/songselect/soundsamples/'.$song[$col].'.mp3"><span class="dashicons dashicons-controls-play"></span></a>';
									break;
								case 'image_link':
									if (trim($song[$col]))
										$text = '<img src="'.$song[$col].'">';
									break;
								case 'first_line':
									$text = '&quot;'.$song[$col].'&hellip;&quot;';
									break;
								case 'ccli_number':
									if (trim($song[$col]))
										$text = '<a target="_blank" href="https://songselect.ccli.com/Songs/'.$song[$col].'"><span class="dashicons dashicons-format-aside"></span></a>';
									break;
								case 'copyright':
									$text = '&copy; '.$song[$col];
									break;
								case 'arrangement_title':
									$text = '('.$song[$col].')';
									break;
								default:
									$text = $song[$col];
							}
						}
						//error_log('subbing "'.$col.'" as "'.$text.'"');
						$row = substr_replace($row, $text, $matches[1][$f][1] - 1, strlen($col) + 2);
					}
					$table_body .= $row;
				} else {
					$table_body .= $content;
				}
				$table_body .= $name_suffix;
			}
			$table_body .= $element_suffix;
		}
	}
	$table_body .= $row_suffix;		
	
	if (!count($songs)) {
		$table_body .= $empty_prefix . 'Nothing found' . $empty_suffix;
	}
	
	$table_body .= $body_suffix;
	
	$output = $prefix . $table_head . $table_body . $suffix;
	
	date_default_timezone_set($old_tz);
	
	return $output;
}

/**
 * Adds Foo_Widget widget.
 */
class ElvantoTools_Roster_Widget extends WP_Widget {

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'elvanto_tools_roster_widget', // Base ID
			__( 'ElvanTools Roster', 'text_domain' ), // Name
			array( 'description' => __( 'Display a roster of services from Elvanto', 'text_domain' ), ) // Args
		);
		wp_register_style('elvantotoolsStylesheet', plugins_url('style.css', __FILE__) );
	    wp_enqueue_style('elvantotoolsStylesheet');
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		//error_log('instance='.print_r($instance, true));
		//error_log('instance='.print_r($instance, true));
		
		echo $args['before_widget'];
		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
		}

		$options = get_option('elvantotools_options');
		//error_log('options='.print_r($options, true));
		
		$system_date_format = 'M j, Y'; 
		$columns = '';
		foreach (explode(',', ELVANTOTOOLS_ROSTER_FIELD_NAMES) as $field) {
			if (isset($instance['column_'.$field]) && $instance['column_'.$field] == 'on') {
				if ($columns != '') {
					$columns .= ',';
				}
				$columns .= $field;
			}
		}
		if ($columns == '') {
			$columns = ELVANTOTOOLS_ROSTER_DEFAULT_WIDGET_COLUMNS;
		}
		
		if (!isset($instance['dates']) || !$instance['dates']) {
			$instance['dates'] = (isset($options['elvanto_date_format']) && $options['elvanto_date_format']) ? 
									$options['elvanto_date_format'] : $system_date_format;
		}
		
		$instance['zone'] = (isset($options['elvanto_time_zone']) && $options['elvanto_time_zone']) ? 
								$options['elvanto_time_zone'] : (get_option('timezone_string', false) ? 
														 	 	get_option('timezone_string') :  
														 		date_default_timezone_get());
		
		
		$instance['key'] = isset($options['elvanto_api_key']) ? $options['elvanto_api_key'] : '';
		$instance['offset'] = isset($options['offset']) ? $options['offset'] : 0;
		if (!isset($instance['weeks'])) $instance['weeks'] = isset($options['elvanto_weeks']) ? $options['elvanto_weeks'] : 1;
		if (!isset($instance['services'])) $instance['services'] = isset($options['elvanto_services']) ? $options['elvanto_services'] : 0;
		if (!isset($instance['service_types'])) $instance['service_types'] = isset($options['roster_service_types']) ? $options['roster_service_types'] : '';
		if (!isset($instance['departments'])) $instance['departments'] = isset($options['roster_departments']) ? $options['roster_departments'] : '';
		if (!isset($instance['sub_departments'])) $instance['sub_departments'] = isset($options['roster_sub_departments']) ? $options['roster_sub_departments'] : '';
		if (!isset($instance['show_details'])) $instance['show_details'] = isset($options['roster_show_details']) ? $options['roster_show_details'] : false;
		if (!isset($instance['positions'])) $instance['positions'] = isset($options['roster_positions']) ? $options['roster_positions'] : '';
		if (!isset($instance['merge'])) $instance['merge'] = isset($options['roster_positions_merge']) ? $options['roster_positions_merge'] : '';
		$show_labels = (!$instance['positions'] || strpos($instance['positions'], ',') !== FALSE);
		$instance['columns'] = $columns;
		if (!isset($instance['sort'])) $instance['sort'] = '';		

		//error_log('instance='.print_r($instance, true));

		
		$display_columns = explode(',',$instance['columns']);
		$content = '['.implode('] [',$display_columns).']';
			
		$prefix = '<div class="elvantotools-widget-list">';
		$suffix = '</div>';
		$top_left_content = '';
		$header_prefix = '<div class="elvantotools-widget-header">';
		$header_suffix = '</div>';
		$header_element_prefix = '<div class="elvantotools-widget-cell">';
		$header_element_suffix = '</div>';
		$header_content = '<strong>[service]</strong>';
		$body_prefix = '<div class="elvantotools-widget-body">';
		$body_suffix = '</div>';
		$label_content = $show_labels ? '<div class="elvantotools-widget-cell elvantotools-widget-label"><strong>[position]</strong></div>' : '';
		$header_label_content = '';
		$row_prefix = '<div class="elvantotools-widget-row">';
		$row_suffix = '</div>';
		$element_prefix = '<div class="elvantotools-widget-cell">';
		$element_suffix = '</div>';
		$name_prefix = '<span name="[id]">';
		$name_suffix = '</span>';
		$name_separator = '<br>';
		$empty_prefix = '<div class="elvantotools-widget-row"><div class="elvantotools-widget-cell">';
		$empty_suffix = '</div></div>';

		$output = elvantotools_roster_generate ($instance['key'], $instance['offset'], $instance['weeks'], $instance['services'], $instance['service_types'], $instance['departments'], $instance['sub_departments'], $instance['positions'], $instance['merge'], $display_columns, $instance['dates'], $instance['zone'], $instance['sort'], $prefix, $suffix, $top_left_content, $header_label_content, $header_content, $header_prefix, $header_suffix, $header_element_prefix, $header_element_suffix, $body_prefix, $body_suffix, $row_prefix, $row_suffix, $label_content, $element_prefix, $element_suffix, $name_prefix, $name_suffix, $name_separator, $content, $empty_prefix, $empty_suffix, $show_details);			
		
		echo do_shortcode($output);
						
		echo $args['after_widget'];
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		//error_log('instance='.print_r($instance, true));
		$options = get_option('elvantotools_options');	
		$title = ! empty( $instance['title'] ) ? $instance['title'] : __( 'Roster', 'text_domain' );
		$key = __( (isset($options['elvanto_api_key']) ? $options['elvanto_api_key'] : ''), 'text_domain' ) ;
		$service_types = ! empty( $instance['service_types'] ) ? $instance['service_types'] : __( (isset($options['roster_service_types']) ? $options['roster_service_types'] : ''), 'text_domain' ) ;
		$departments = ! empty( $instance['departments'] ) ? $instance['departments'] : __( (isset($options['roster_departments']) ? $options['roster_departments'] : ''), 'text_domain' ) ;
		$sub_departments = ! empty( $instance['sub_departments'] ) ? $instance['sub_departments'] : __( (isset($options['roster_sub_departments']) ? $options['roster_sub_departments'] : ''), 'text_domain' ) ;
		$positions = ! empty( $instance['positions'] ) ? $instance['positions'] : __( (isset($options['roster_positions']) ? $options['roster_positions'] : ''), 'text_domain' ) ;
		$merge = ! empty( $instance['merge'] ) ? $instance['merge'] : __( (isset($options['roster_positions_merge']) ? $options['roster_positions_merge'] : ''), 'text_domain' ) ;
		$offset = ! empty( $instance['offset'] ) ? $instance['offset'] : __( (isset($options['offset']) ? $options['offset'] : '0'), 'text_domain' );
		$weeks = ! empty( $instance['weeks'] ) ? $instance['weeks'] : __( (isset($options['elvanto_weeks']) ? $options['elvanto_weeks'] : '1'), 'text_domain' );
		$services = ! empty( $instance['services'] ) ? $instance['services'] : __( (isset($options['elvanto_services']) ? $options['elvanto_services'] : '0'), 'text_domain' );
		$dates = ! empty( $instance['dates'] ) ? $instance['dates'] : __( '', 'text_domain' );
		$show_details = ! empty( $instance['show_details'] ) ? $instance['show_details'] : __( (isset($options['roster_show_details']) ? $options['roster_show_details'] : false), 'text_domain' );		
		?>
		<div class="elvantotools-widget-header-logo">
          	<a href="https://elvanto.com"><img src="<?=plugins_url('ElvantoToolsPPHeader.png', __FILE__)?>"></a>
        </div>
        <br>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'service_types' ); ?>"><?php _e( 'Service Types:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'service_types' ); ?>" name="<?php echo $this->get_field_name( 'service_types' ); ?>" type="text" value="<?php echo esc_attr( $service_types ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'departments' ); ?>"><?php _e( 'Departments:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'departments' ); ?>" name="<?php echo $this->get_field_name( 'departments' ); ?>" type="text" value="<?php echo esc_attr( $departments ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'sub_departments' ); ?>"><?php _e( 'Sub-departments:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'sub_departments' ); ?>" name="<?php echo $this->get_field_name( 'sub_departments' ); ?>" type="text" value="<?php echo esc_attr( $sub_departments ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'positions' ); ?>"><?php _e( 'Positions:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'positions' ); ?>" name="<?php echo $this->get_field_name( 'positions' ); ?>" type="text" value="<?php echo esc_attr( $positions ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'merge' ); ?>"><?php _e( 'Merge:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'merge' ); ?>" name="<?php echo $this->get_field_name( 'merge' ); ?>" type="text" value="<?php echo esc_attr( $merge ); ?>">
		</p>
        <table class="elvantotools-widget"><tbody>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'weeks' ); ?>"><?php _e( 'Weeks ahead:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'weeks' ); ?>" name="<?php echo $this->get_field_name( 'weeks' ); ?>" type="number" value="<?php echo esc_attr( $weeks ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'services' ); ?>"><?php _e( 'Services ahead:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'services' ); ?>" name="<?php echo $this->get_field_name( 'services' ); ?>" type="number" value="<?php echo esc_attr( $services ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'dates' ); ?>"><a href="http://php.net/manual/en/function.date.php">Date format</a></label></td>
			<td><input class="elvantotools-widget-input-narrow" id="<?php echo $this->get_field_id( 'dates' ); ?>" name="<?php echo $this->get_field_name( 'dates' ); ?>" type="text" value="<?php echo esc_attr( $dates ); ?>" placeholder="e.g. M j, Y">&nbsp;&nbsp;</td>
		</tr>        
        <tr>
			<td><label for="<?php echo $this->get_field_id('show_details'); ?>"><input type="checkbox" id="<?php echo $this->get_field_id('show_details');?>" name="<?php echo $this->get_field_name('show_details');?>" <?=(isset($instance['show_details']) && $instance['show_details'] == 'on') ? 'CHECKED' : ''?>>&nbsp;<?php _e( 'Link to details'); ?></label></td>
		</tr>        
        </tbody></table>
        <div class="elvantotools-widget">
        <label class="elvantotools-widget-label"><?php _e( 'Columns:' ); ?></label>
            <div class="elvantotools-widget-settings-list">
            <?php foreach (explode(',',ELVANTOTOOLS_ROSTER_FIELD_NAMES) as $field) { $column = 'column_' . $field; ?>
                <label for="<?php echo $this->get_field_id($column); ?>"><input type="checkbox" id="<?php echo $this->get_field_id($column);?>" name="<?php echo $this->get_field_name($column);?>" <?=(isset($instance[$column]) && $instance[$column] == 'on') ? 'CHECKED' : ''?>>&nbsp;<?php _e( ucwords(str_replace('_',' ',$field))); ?></label><br>
            <?php } ?>
            </div>
        </div>
		<?php 
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		//error_log('new_instance='.print_r($new_instance, true));
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['key'] = ( ! empty( $new_instance['key'] ) ) ? $new_instance['key'] : '';
		$instance['service_types'] = ( ! empty( $new_instance['service_types'] ) ) ? $new_instance['service_types'] : '';
		$instance['show_details'] = ( ! empty( $new_instance['show_details'] ) ) ? $new_instance['show_details'] : false;				
		$instance['departments'] = ( ! empty( $new_instance['departments'] ) ) ? $new_instance['departments'] : '';
		$instance['sub_departments'] = ( ! empty( $new_instance['sub_departments'] ) ) ? $new_instance['sub_departments'] : '';
		$instance['positions'] = ( ! empty( $new_instance['positions'] ) ) ? $new_instance['positions'] : '';
		$instance['merge'] = ( ! empty( $new_instance['merge'] ) ) ? $new_instance['merge'] : '';
		$instance['offset'] = ( ! empty( $new_instance['offset'] ) ) ? absint($new_instance['offset']) : 0;
		$instance['weeks'] = ( ! empty( $new_instance['weeks'] ) ) ? $new_instance['weeks'] : 1;
		$instance['services'] = ( ! empty( $new_instance['services']) && $new_instance['services']) ? $new_instance['services'] : '';
		$instance['dates'] = ( ! empty( $new_instance['dates'] ) ) ? $new_instance['dates'] : '';
		foreach (explode(',',ELVANTOTOOLS_ROSTER_FIELD_NAMES) as $field) {
			$column = 'column_' . $field;
			$instance[$column] = ( ! empty( $new_instance[$column] ) ) ? $new_instance[$column] : '';
		}
		return $instance;
	}

} // class ElvantoTools_Roster_Widget

/**
 * Adds Foo_Widget widget.
 */
/**
 * Adds Foo_Widget widget.
 */
class ElvantoTools_Songs_Widget extends WP_Widget {

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'elvanto_tools_songs_widget', // Base ID
			__( 'ElvanTools Songs', 'text_domain' ), // Name
			array( 'description' => __( 'Display service song lists from Elvanto', 'text_domain' ), ) // Args
		);
		wp_register_style('elvantotoolsStylesheet', plugins_url('style.css', __FILE__) );
	    wp_enqueue_style('elvantotoolsStylesheet');
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {
		//error_log('instance='.print_r($instance, true));
		//error_log('instance='.print_r($instance, true));
		
		echo $args['before_widget'];
		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ). $args['after_title'];
		}

		$options = get_option('elvantotools_options');
		//error_log('options='.print_r($options, true));
		
		$system_date_format = 'M j, Y'; 
		$columns = '';
		foreach (explode(',', ELVANTOTOOLS_SONGS_FIELD_NAMES) as $field) {
			if (isset($instance['column_'.$field]) && $instance['column_'.$field] == 'on') {
				if ($columns != '') {
					$columns .= ',';
				}
				$columns .= $field;
			}
		}
		if ($columns == '') {
			$columns = ELVANTOTOOLS_SONGS_DEFAULT_WIDGET_COLUMNS;
		}
		
		if (!isset($instance['dates']) || !$instance['dates']) {
			$instance['dates'] = (isset($options['elvanto_date_format']) && $options['elvanto_date_format']) ? 
									$options['elvanto_date_format'] : $system_date_format;
		}
		
		$instance['zone'] = (isset($options['elvanto_time_zone']) && $options['elvanto_time_zone']) ? 
								$options['elvanto_time_zone'] : (get_option('timezone_string', false) ? 
														 	 	get_option('timezone_string') :  
														 		date_default_timezone_get());
		
		
		$instance['key'] = isset($options['elvanto_api_key']) ? $options['elvanto_api_key'] : '';
		$instance['offset'] = isset($options['offset']) ? $options['offset'] : 0;
		$instance['in_plan'] = isset($options['song_in_plan']) ? $options['song_in_plan'] : false;
		if (!isset($instance['weeks'])) $instance['weeks'] = isset($options['elvanto_weeks']) ? $options['elvanto_weeks'] : 1;
		if (!isset($instance['services'])) $instance['services'] = isset($options['elvanto_services']) ? $options['elvanto_services'] : 0;
		if (!isset($instance['video_height'])) $instance['video_height'] = isset($options['song_video_height']) ? $options['song_video_height'] : 0;
		if (!isset($instance['video_width'])) $instance['video_width'] = isset($options['song_video_width']) ? $options['song_video_width'] : 0;
		if (!isset($instance['service_types'])) $instance['service_types'] = isset($options['songs_service_types']) ? $options['songs_service_types'] : '';
		if (!isset($instance['categories'])) $instance['categories'] = isset($options['songs_categories']) ? $options['songs_categories'] : '';
		$instance['columns'] = $columns;
		if (!isset($instance['sort'])) $instance['sort'] = '';		

		//error_log('instance='.print_r($instance, true));

				
		$display_columns = explode(',',$instance['columns']);
		$content = '';
		foreach ($display_columns as $col) $content .= '<span class="elvantotools_song_'.$col.'">['.$col.']</span>';
			
		$prefix = '<div class="elvantotools-widget-list">';
		$suffix = '</div>';
		$top_left_content = '';
		$header_prefix = '<div class="elvantotools-widget-header">';
		$header_suffix = '</div>';
		$header_element_prefix = '<div class="elvantotools-widget-cell">';
		$header_element_suffix = '</div>';
		$header_content = '<strong>[service]</strong>';
		$body_prefix = '<div class="elvantotools-widget-body">';
		$body_suffix = '</div>';
		$label_content = '';
		$header_label_content = '';
		$row_prefix = '<div class="elvantotools-widget-row">';
		$row_suffix = '</div>';
		$element_prefix = '<div class="elvantotools-widget-cell">';
		$element_suffix = '</div>';
		$name_prefix = '<div class="elvantotools-widget-song" name=[id]>';
		$name_suffix = '</div>';
		$name_separator = '';
		$empty_prefix = '<div class="elvantotools-widget-row"><div class="elvantotools-widget-cell">';
		$empty_suffix = '</div></div>';

		$output = elvantotools_songs_generate ($instance['key'], $instance['offset'], $instance['weeks'], $instance['services'], $instance['service_types'], $instance['categories'], $display_columns, $instance['dates'], $instance['zone'], $instance['sort'], $prefix, $suffix, $top_left_content, $header_label_content, $header_content, $header_prefix, $header_suffix, $header_element_prefix, $header_element_suffix, $body_prefix, $body_suffix, $row_prefix, $row_suffix, $label_content, $element_prefix, $element_suffix, $name_prefix, $name_suffix, $name_separator, $content, $empty_prefix, $empty_suffix,$instance['video_width'],$instance['video_height'],$instance['in_plan']);			
		
		echo do_shortcode($output);
						
		echo $args['after_widget'];
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {
		//error_log('instance='.print_r($instance, true));
		$options = get_option('elvantotools_options');	
		$title = ! empty( $instance['title'] ) ? $instance['title'] : __( 'Song List', 'text_domain' );
		$key = __( (isset($options['elvanto_api_key']) ? $options['elvanto_api_key'] : ''), 'text_domain' ) ;
		$service_types = ! empty( $instance['service_types'] ) ? $instance['service_types'] : __( (isset($options['roster_service_types']) ? $options['roster_service_types'] : ''), 'text_domain' ) ;
		$categories = ! empty( $instance['categories'] ) ? $instance['categories'] : __( (isset($options['songs_categories']) ? $options['songs_categories'] : ''), 'text_domain' ) ;
		$offset = ! empty( $instance['offset'] ) ? $instance['offset'] : __( (isset($options['offset']) ? $options['offset'] : '0'), 'text_domain' );
		$weeks = ! empty( $instance['weeks'] ) ? $instance['weeks'] : __( (isset($options['elvanto_weeks']) ? $options['elvanto_weeks'] : '1'), 'text_domain' );
		$video_height = ! empty( $instance['video_height'] ) ? $instance['video_height'] : __( (isset($options['song_video_height']) ? $options['song_video_height'] : '0'), 'text_domain' );
		$video_width = ! empty( $instance['video_width'] ) ? $instance['video_width'] : __( (isset($options['song_video_width']) ? $options['song_video_width'] : '0'), 'text_domain' );
		$services = ! empty( $instance['services'] ) ? $instance['services'] : __( (isset($options['elvanto_services']) ? $options['elvanto_services'] : '0'), 'text_domain' );
		$dates = ! empty( $instance['dates'] ) ? $instance['dates'] : __( '', 'text_domain' );
		$in_plan = ! empty( $instance['in_plan'] ) ? $instance['in_plan'] : __( (isset($options['song_in_plan']) ? $options['song_in_plan'] : false), 'text_domain' );
		?>
		<div class="elvantotools-widget-header-logo">
          	<a href="https://elvanto.com"><img src="<?=plugins_url('ElvantoToolsPPHeader.png', __FILE__)?>"></a>
        </div>
        <br>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( 'Title:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" type="text" value="<?php echo esc_attr( $title ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'service_types' ); ?>"><?php _e( 'Service Types:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'service_types' ); ?>" name="<?php echo $this->get_field_name( 'service_types' ); ?>" type="text" value="<?php echo esc_attr( $service_types ); ?>">
		</p>
		<p class="elvantotools-widget">
		<label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'departments' ); ?>"><?php _e( 'Categories:' ); ?></label> 
		<input class="widefat" id="<?php echo $this->get_field_id( 'categories' ); ?>" name="<?php echo $this->get_field_name( 'categories' ); ?>" type="text" value="<?php echo esc_attr( $categories ); ?>">
		</p>
        <table class="elvantotools-widget"><tbody>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'weeks' ); ?>"><?php _e( 'Weeks ahead:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'weeks' ); ?>" name="<?php echo $this->get_field_name( 'weeks' ); ?>" type="number" value="<?php echo esc_attr( $weeks ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'services' ); ?>"><?php _e( 'Services ahead:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'services' ); ?>" name="<?php echo $this->get_field_name( 'services' ); ?>" type="number" value="<?php echo esc_attr( $services ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'video_width' ); ?>"><?php _e( 'Video width:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'video_width' ); ?>" name="<?php echo $this->get_field_name( 'video_width' ); ?>" type="number" value="<?php echo esc_attr( $video_width ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'video_height' ); ?>"><?php _e( 'Video height:' ); ?></label></td>
			<td><input id="<?php echo $this->get_field_id( 'video_height' ); ?>" name="<?php echo $this->get_field_name( 'video_height' ); ?>" type="number" value="<?php echo esc_attr( $video_height ); ?>"></td>
		</tr>
        <tr>
        	<td><label class="elvantotools-widget-label" for="<?php echo $this->get_field_id( 'dates' ); ?>"><a href="http://php.net/manual/en/function.date.php">Date format</a></label></td>
			<td><input class="elvantotools-widget-input-narrow" id="<?php echo $this->get_field_id( 'dates' ); ?>" name="<?php echo $this->get_field_name( 'dates' ); ?>" type="text" value="<?php echo esc_attr( $dates ); ?>" placeholder="e.g. M j, Y">&nbsp;&nbsp;</td>
		</tr>        
        </tbody></table>
        <div class="elvantotools-widget">
        <label class="elvantotools-widget-label"><?php _e( 'Columns:' ); ?></label>
            <div class="elvantotools-widget-settings-list">
            <?php foreach (explode(',',ELVANTOTOOLS_SONGS_FIELD_NAMES) as $field) { $column = 'column_' . $field; ?>
                <label for="<?php echo $this->get_field_id($column); ?>"><input type="checkbox" id="<?php echo $this->get_field_id($column);?>" name="<?php echo $this->get_field_name($column);?>" <?=(isset($instance[$column]) && $instance[$column] == 'on') ? 'CHECKED' : ''?>>&nbsp;<?php _e(ucwords(str_replace('_',' ',$field))); ?></label><br>
            <?php } ?>
            </div>
        </div>
		<?php 
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();
		//error_log('new_instance='.print_r($new_instance, true));
		$instance['title'] = ( ! empty( $new_instance['title'] ) ) ? strip_tags( $new_instance['title'] ) : '';
		$instance['key'] = ( ! empty( $new_instance['key'] ) ) ? $new_instance['key'] : '';
		$instance['in_plan'] = ( ! empty( $new_instance['in_plan'] ) ) ? $new_instance['in_plan'] : false;		
		$instance['service_types'] = ( ! empty( $new_instance['service_types'] ) ) ? $new_instance['service_types'] : '';
		$instance['categories'] = ( ! empty( $new_instance['categories'] ) ) ? $new_instance['categories'] : '';
		$instance['offset'] = ( ! empty( $new_instance['offset'] ) ) ? absint($new_instance['offset']) : 0;
		$instance['weeks'] = ( ! empty( $new_instance['weeks'] ) ) ? $new_instance['weeks'] : 1;
		$instance['video_width'] = ( ! empty( $new_instance['video_width'] ) ) ? absint($new_instance['video_width']) : 0;
		$instance['video_height'] = ( ! empty( $new_instance['video_height'] ) ) ? absint($new_instance['video_height']) : 0;
		$instance['services'] = ( ! empty( $new_instance['services']) && $new_instance['services']) ? $new_instance['services'] : '';
		$instance['dates'] = ( ! empty( $new_instance['dates'] ) ) ? $new_instance['dates'] : '';
		foreach (explode(',',ELVANTOTOOLS_SONGS_FIELD_NAMES) as $field) {
			$column = 'column_' . $field;
			$instance[$column] = ( ! empty( $new_instance[$column] ) ) ? $new_instance[$column] : '';
		}
		return $instance;
	}

} // class ElvantoTools_Songs_Widget
?>